import sfml as sf

class LShape(sf.Shape):
    '''Simple container class to store a set of vertices.'''
    def __init__(self, vertices):
        sf.Shape.__init__(self)
        self.vertices = vertices
        
        def close_enough((x1, y1), (x2, y2)):
            return abs(x1 - x2) + abs(y1 - y2) < 0.001
        self.closed = close_enough(self.vertices[0],self.vertices[-1])
        
    def get_point(self, index):
        '''Overloaded function returning appropriate vertex.'''
        if index < len(self.vertices):
            return self.vertices[index]
        else:
            return self.vertices[len(self.vertices) - index - 1]
    
    def get_point_count(self):
        '''Overloaded function returning vertex count.'''
        if self.closed:
            return len(self.vertices)
        return len(self.vertices)*2
    
    def fit_view(self, target):
        '''Based on vertex positions, calculates and sets an appropriate view.'''
        xmin = min(v[0] for v in self.vertices)
        xmax = max(v[0] for v in self.vertices)
        ymin = min(v[1] for v in self.vertices)
        ymax = max(v[1] for v in self.vertices)
        
        border = 5
        xmin -= border
        xmax += border
        ymin -= border
        ymax += border
        
        aspect_ratio = target.width / float(target.height)
        width  = xmax - xmin
        height = ymax - ymin

        if width / height > aspect_ratio:
            view_height = width / aspect_ratio
            height_gap = view_height - height
            view = sf.View.from_rect(
                sf.FloatRect(xmin, ymax - height_gap/2,
                             width, -view_height))
        else:
            view_width = height * aspect_ratio
            width_gap = view_width - width
            view = sf.View.from_rect(
                sf.FloatRect(xmin - width_gap/2, ymax,
                             view_width, -height))
        return view
