'''Functions to grow and evaluate an L-system.'''

def build_string(string, rules, iterations):
    '''Recursively builds an L-system string, returning an generator.'''
    if iterations == 0:
        for char in string:
            yield char
    else:
        for char in string:
            if char in rules.keys():
                for c in build_string(rules[char], rules, iterations - 1):
                    yield c
            else:
                yield char

def build_shape(string, functions, (x, y, a) = (0, 0, 0)):
    '''Generate a set of vertices from an L-system generator and rules.'''
    vertices = [(x, y)]
    for c in string:
        if c in functions.keys():
            x, y, a = functions[c](x, y, a)
            if (x, y) != vertices[-1]:
                vertices += [(x, y)]
    return vertices